@@ -5,6 +5,8 @@
  * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
+#include "database.h"
+
 #include <QDateTime>
 #include <QDebug>
 #include <QDir>
@@ -15,8 +17,6 @@
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
-#include "database.h"
-
 #define TRUE_OR_RETURN(x)                                                                                                                                      \
     if (!x)                                                                                                                                                    \
         return false;
@@ -47,12 +47,17 @@ bool Database::migrate()
         TRUE_OR_RETURN(migrateTo3());
     if (dbversion < 4)
         TRUE_OR_RETURN(migrateTo4());
+    if (dbversion < 5)
+        TRUE_OR_RETURN(migrateTo5());
+    if (dbversion < 6)
+        TRUE_OR_RETURN(migrateTo6());
     return true;
 }
 
 bool Database::migrateTo1()
 {
     qDebug() << "Migrating database to version 1";
+    TRUE_OR_RETURN(transaction());
     TRUE_OR_RETURN(
         execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Feeds (name TEXT, url TEXT, image TEXT, link TEXT, description TEXT, deleteAfterCount INTEGER, "
                                "deleteAfterType INTEGER, subscribed INTEGER, lastUpdated INTEGER, new BOOL, notify BOOL);")));
@@ -66,24 +71,25 @@ bool Database::migrateTo1()
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Queue (listnr INTEGER, feed TEXT, id TEXT, playing BOOL);")));
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Errors (url TEXT, id TEXT, code INTEGER, message TEXT, date INTEGER);")));
     TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 1;")));
+    TRUE_OR_RETURN(commit());
     return true;
 }
 
 bool Database::migrateTo2()
 {
     qDebug() << "Migrating database to version 2";
-    TRUE_OR_RETURN(execute(QStringLiteral("BEGIN TRANSACTION;")));
+    TRUE_OR_RETURN(transaction());
     TRUE_OR_RETURN(execute(QStringLiteral("DROP TABLE Errors;")));
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Errors (type INTEGER, url TEXT, id TEXT, code INTEGER, message TEXT, date INTEGER);")));
     TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 2;")));
-    TRUE_OR_RETURN(execute(QStringLiteral("COMMIT;")));
+    TRUE_OR_RETURN(commit());
     return true;
 }
 
 bool Database::migrateTo3()
 {
     qDebug() << "Migrating database to version 3";
-    TRUE_OR_RETURN(execute(QStringLiteral("BEGIN TRANSACTION;")));
+    TRUE_OR_RETURN(transaction());
     TRUE_OR_RETURN(
         execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Enclosurestemp (feed TEXT, id TEXT, duration INTEGER, size INTEGER, title TEXT, type TEXT, url "
                                "TEXT, playposition INTEGER, downloaded INTEGER);")));
@@ -94,19 +100,43 @@ bool Database::migrateTo3()
     TRUE_OR_RETURN(execute(QStringLiteral("DROP TABLE Enclosures;")));
     TRUE_OR_RETURN(execute(QStringLiteral("ALTER TABLE Enclosurestemp RENAME TO Enclosures;")));
     TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 3;")));
-    TRUE_OR_RETURN(execute(QStringLiteral("COMMIT;")));
+    TRUE_OR_RETURN(commit());
     return true;
 }
 
 bool Database::migrateTo4()
 {
     qDebug() << "Migrating database to version 4";
-    TRUE_OR_RETURN(execute(QStringLiteral("BEGIN TRANSACTION;")));
+    TRUE_OR_RETURN(transaction());
     TRUE_OR_RETURN(execute(QStringLiteral("DROP TABLE Errors;")));
     TRUE_OR_RETURN(
         execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Errors (type INTEGER, url TEXT, id TEXT, code INTEGER, message TEXT, date INTEGER, title TEXT);")));
     TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 4;")));
-    TRUE_OR_RETURN(execute(QStringLiteral("COMMIT;")));
+    TRUE_OR_RETURN(commit());
+    return true;
+}
+
+bool Database::migrateTo5()
+{
+    qDebug() << "Migrating database to version 5";
+    TRUE_OR_RETURN(transaction());
+    TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Chapters (feed TEXT, id TEXT, start INTEGER, title TEXT, link TEXT, image TEXT);")));
+    TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 5;")));
+    TRUE_OR_RETURN(commit());
+    return true;
+}
+
+bool Database::migrateTo6()
+{
+    qDebug() << "Migrating database to version 6";
+    TRUE_OR_RETURN(transaction());
+    TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS SyncTimestamps (syncservice TEXT, timestamp INTEGER);")));
+    TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS FeedActions (url TEXT, action TEXT, timestamp INTEGER);")));
+    TRUE_OR_RETURN(
+        execute(QStringLiteral("CREATE TABLE IF NOT EXISTS EpisodeActions (podcast TEXT, url TEXT, id TEXT, action TEXT, started INTEGER, position INTEGER, "
+                               "total INTEGER, timestamp INTEGER);")));
+    TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 6;")));
+    TRUE_OR_RETURN(commit());
     return true;
 }
 
@@ -128,6 +158,16 @@ bool Database::execute(QSqlQuery &query)
     return true;
 }
 
+bool Database::transaction()
+{
+    return QSqlDatabase::database().transaction();
+}
+
+bool Database::commit()
+{
+    return QSqlDatabase::database().commit();
+}
+
 int Database::version()
 {
     QSqlQuery query;
