@@ -32,10 +32,10 @@ class Entry : public QObject
     Q_PROPERTY(QDateTime updated READ updated CONSTANT)
     Q_PROPERTY(QString link READ link CONSTANT)
     Q_PROPERTY(QString baseUrl READ baseUrl CONSTANT)
-    Q_PROPERTY(bool read READ read WRITE setRead NOTIFY readChanged);
-    Q_PROPERTY(bool new READ getNew WRITE setNew NOTIFY newChanged);
-    Q_PROPERTY(Enclosure *enclosure READ enclosure CONSTANT);
-    Q_PROPERTY(bool hasEnclosure READ hasEnclosure CONSTANT);
+    Q_PROPERTY(bool read READ read WRITE setRead NOTIFY readChanged)
+    Q_PROPERTY(bool new READ getNew WRITE setNew NOTIFY newChanged)
+    Q_PROPERTY(Enclosure *enclosure READ enclosure CONSTANT)
+    Q_PROPERTY(bool hasEnclosure READ hasEnclosure CONSTANT)
     Q_PROPERTY(QString image READ image WRITE setImage NOTIFY imageChanged)
     Q_PROPERTY(QString cachedImage READ cachedImage NOTIFY cachedImageChanged)
     Q_PROPERTY(bool queueStatus READ queueStatus WRITE setQueueStatus NOTIFY queueStatusChanged)
@@ -69,6 +69,10 @@ public:
 
     Q_INVOKABLE QString adjustedContent(int width, int fontSize);
 
+    void setNewInternal(bool state);
+    void setReadInternal(bool read);
+    void setQueueStatusInternal(bool state);
+
 Q_SIGNALS:
     void readChanged(bool read);
     void newChanged(bool state);
